''' <summary>
''' This business layer class representing a single Album.
''' </summary>
''' <remarks></remarks>
Public Class Album

    Private _Name As String
    Private _Artist As String
    Private _JacketArt As String
    Private _Songs As List(Of Song)
    Private _Index As Integer


    ''' <summary>
    ''' This constructor takes parameters for all properties except the list of Songs.  That list
    ''' needs to be initialized through the Songs property.
    ''' </summary>
    ''' <param name="NewAlbumName">The name of the new Album object.</param>
    ''' <param name="NewArtistName">The artist attached to the new Album</param>
    ''' <param name="NewJacketArtPath">The path to the album art image file</param>
    ''' <param name="AlbumIndex">The index in the container</param>
    ''' <remarks></remarks>
    Public Sub New(ByVal NewAlbumName As String, ByVal NewArtistName As String, ByVal NewJacketArtPath As String, ByVal AlbumIndex As Integer)
        Me._Name = NewAlbumName
        Me._Artist = NewArtistName
        Me._JacketArt = NewJacketArtPath
        Me._Songs = New List(Of Song)
        Me._Index = AlbumIndex
    End Sub


    ''' <summary>
    ''' This exposes the Album object's name.
    ''' </summary>
    ''' <value></value>
    ''' <returns>Returns the Name of the Album, represented as a string</returns>
    ''' <remarks></remarks>
    Public ReadOnly Property Name() As String
        Get
            Return Me._Name
        End Get
    End Property


    ''' <summary>
    ''' This exposes the Album object's artist.
    ''' </summary>
    ''' <value></value>
    ''' <returns>Returns the Artist of the Album, represented as a string</returns>
    ''' <remarks></remarks>
    Public ReadOnly Property Artist() As String
        Get
            Return Me._Artist
        End Get
    End Property


    ''' <summary>
    ''' This exposes the Album object's jacket art file path.
    ''' </summary>
    ''' <value></value>
    ''' <returns>Returns the path to the Jacket Art image file for the Album, represented as a string</returns>
    ''' <remarks></remarks>
    Public ReadOnly Property JacketArtPath() As String
        Get
            Return Me._JacketArt
        End Get
    End Property


    ''' <summary>
    ''' This exposes the Album object's list of Songs.
    ''' </summary>
    ''' <value></value>
    ''' <returns>Gets or sets the list of Songs for the Album</returns>
    ''' <remarks></remarks>
    Public Property Songs() As List(Of Song)
        Get
            Return Me._Songs
        End Get
        Set(ByVal value As List(Of Song))
            Me._Songs = value
        End Set
    End Property


    ''' <summary>
    ''' This gets or sets the index of the album on the Shelf.
    ''' </summary>
    ''' <value></value>
    ''' <returns>The index of the album on the Shelf.</returns>
    ''' <remarks></remarks>
    Public Property Index() As Integer
        Get
            Return Me._Index
        End Get
        Set(ByVal value As Integer)
            Me._Index = value
        End Set
    End Property
End Class
