''' <summary>
''' This business layer class representing a single Song.
''' </summary>
''' <remarks></remarks>
Public Class Song

    Private _Name As String
    Private _PathToMedia As String
    Private _Index As Integer

    ''' <summary>
    ''' This constructor takes all elements of a new Song object as parameters.
    ''' </summary>
    ''' <param name="NewSongName">The name of the Song</param>
    ''' <param name="PathToMedia">The file system path to the media.</param>
    ''' <param name="SongIndex">The index in the Song's container.</param>
    ''' <remarks></remarks>
    Public Sub New(ByVal NewSongName As String, ByVal PathToMedia As String, ByVal SongIndex As Integer)
        Me._Name = NewSongName
        Me._PathToMedia = PathToMedia
        Me._Index = SongIndex
    End Sub

#Region " Properties "
    ''' <summary>
    ''' This exposes the Name of the Song
    ''' </summary>
    ''' <value></value>
    ''' <returns>The Name of the Song, represented as a string</returns>
    ''' <remarks></remarks>
    Public ReadOnly Property Name() As String
        Get
            Return Me._Name
        End Get
    End Property


    ''' <summary>
    ''' This exposes the Path to the actual media of the Song
    ''' </summary>
    ''' <value></value>
    ''' <returns>The Path to the actual media, represented as a string</returns>
    ''' <remarks></remarks>
    Public ReadOnly Property PathToMedia() As String
        Get
            ' TODO: This should return "" if there isn't a song loaded
            Return Me._PathToMedia
        End Get
    End Property


    ''' <summary>
    ''' This gets or sets the Index property (the position of this song within an album)
    ''' </summary>
    ''' <value></value>
    ''' <returns>The position of the song within the album</returns>
    ''' <remarks></remarks>
    Public Property Index() As Integer
        Get
            Return Me._Index
        End Get
        Set(ByVal value As Integer)
            Me._Index = value
        End Set
    End Property
#End Region


#Region " Methods "
    ''' <summary>
    ''' This overrides the base ToString function to return the name of the song selected.
    ''' </summary>
    ''' <returns>The Name of the Song, represented as a string.</returns>
    ''' <remarks></remarks>
    Public Overrides Function ToString() As String
        Return Me._Name
    End Function
#End Region


End Class
