﻿Imports System.Configuration

''' <summary>
''' This encapsulates the core application settings.  The members are declared as Shared intentionally
''' to make these values accessible as essentially global variables.
''' </summary>
''' <remarks></remarks>
Public Class Core

    Shared _MusicBasePath As String
    Shared _MetaDataMasterFileName As String
    Shared _MetaDataPath As String
    Shared _IsLoaded As Boolean

    ''' <summary>
    ''' This method checks to see if the settings have already been loaded, and if so simply returns.
    ''' If not, it loads the settings from the config file and then returns.
    ''' </summary>
    ''' <remarks></remarks>
    Private Shared Sub LoadSettings()
        Dim MyConfigFile As Configuration
        Dim MyAppSettings As AppSettingsSection

        If (_IsLoaded) Then Return

        ' This works because I've named the file WPFMediaPlayer.AppSettings.dll.config, and marked it
        ' to always copy itself to the executable directory.
        '
        ' If I called the file something like "Core.config", I would need to use "Core.config" here,
        ' which would get past the Open... function, but then fail when it gets to the GetSection
        ' method because it would be looking for a file called "Core.config.config".  If I try to use
        ' simply "Core", it fails on the Open... function because that file doesn't exist.
        MyConfigFile = ConfigurationManager.OpenExeConfiguration("WPFMediaPlayer.AppSettings.dll")
        MyAppSettings = CType(MyConfigFile.GetSection("appSettings"), AppSettingsSection)

        ' Load the values
        _MusicBasePath = MyAppSettings.Settings.Item("MusicBasePath").Value
        _MetaDataMasterFileName = MyAppSettings.Settings.Item("MetaDataMasterFileName").Value
        _MetaDataPath = MyAppSettings.Settings.Item("MetaDataPath").Value

        _IsLoaded = True
    End Sub


#Region " Properties "
    ''' <summary>
    ''' This exposes the MusicBasePath value
    ''' </summary>
    ''' <value>String</value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public Shared ReadOnly Property MusicBasePath() As String
        Get
            LoadSettings()
            Return _MusicBasePath
        End Get
    End Property


    ''' <summary>
    ''' This exposes the MetaDataMasterFileName value
    ''' </summary>
    ''' <value>String</value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public Shared ReadOnly Property MetaDataMasterFileName() As String
        Get
            LoadSettings()
            Return _MetaDataMasterFileName
        End Get
    End Property


    ''' <summary>
    ''' This exposes the MetaDataPath value
    ''' </summary>
    ''' <value>String</value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public Shared ReadOnly Property MetaDataPath() As String
        Get
            LoadSettings()
            Return _MetaDataPath
        End Get
    End Property
#End Region



End Class
