﻿Imports WPFMediaPlayer.BusinessObjects

''' <summary>
''' This form is used to browse the songs from the album selected
''' </summary>
''' <remarks></remarks>
Partial Public Class AlbumSongs


    Public Event SongSelected(ByVal NewSong As Song)
    Public Event SongHighlighted(ByVal NewSong As Song)
    Public Event AlbumSongsClosed()


    ''' <summary>
    ''' This initializes the form with the album passed in
    ''' </summary>
    ''' <param name="AlbumToLoad"></param>
    ''' <remarks></remarks>
    Public Sub New(ByVal AlbumToLoad As Album)
        ' This call is required by the Windows Form Designer.
        InitializeComponent()

        Me.RefreshSongList(AlbumToLoad, Nothing)
    End Sub



#Region " Properties "
    ''' <summary>
    ''' This exposes the item currently selected in the SongList drop down, and returns the corresponding Song object.
    ''' </summary>
    ''' <value></value>
    ''' <returns>This returns a Song object for the item selected, or Nothing if no item is selected.</returns>
    ''' <remarks></remarks>
    Public ReadOnly Property CurrentSong() As Song
        Get
            If (Me.SongList.SelectedItem Is Nothing) Then Return Nothing

            Return CType(Me.SongList.SelectedItem, Song)
        End Get
    End Property
#End Region

#Region " Methods "
    ''' <summary>
    ''' This closes the form.
    ''' </summary>
    ''' <remarks></remarks>
    Private Sub CloseForm()
        Me.Close()
    End Sub


    ''' <summary>
    ''' This loads up the drop down list with the album's songs
    ''' </summary>
    ''' <param name="AlbumToLoad"></param>
    ''' <remarks></remarks>
    Public Sub RefreshSongList(ByVal AlbumToLoad As Album, ByVal SongToSelect As Song)
        Dim NewAlbumSong As Song

        Me.SongList.Items.Clear()
        For Each NewAlbumSong In AlbumToLoad.Songs
            Me.SongList.Items.Add(NewAlbumSong)

            If (NewAlbumSong.Equals(SongToSelect)) Then Me.SongList.SelectedIndex = Me.SongList.Items.Count - 1
        Next
    End Sub


    ''' <summary>
    ''' This overloaded routine allows the calling window to pass in a reference to itself so that the current
    ''' window can position itself relative to it.
    ''' </summary>
    ''' <param name="ParentForm"></param>
    ''' <remarks></remarks>
    Public Overloads Sub Show(ByVal ParentForm As System.Windows.Window)
        Me.Left = ParentForm.Left + 50
        Me.Top = ParentForm.Top + 50
        MyBase.ShowDialog()
    End Sub
#End Region


#Region " Event Handlers "
    ''' <summary>
    ''' This allows the user to drag and move the window.
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Private Sub AlbumSongs_MouseDown(ByVal sender As Object, ByVal e As System.Windows.Input.MouseButtonEventArgs) Handles Me.MouseDown
        Me.DragMove()
    End Sub


    ''' <summary>
    ''' This closes the form.
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Private Sub CloseButton_Click(ByVal sender As Object, ByVal e As System.Windows.RoutedEventArgs) Handles CloseButton.Click
        CloseForm()
    End Sub


    ''' <summary>
    ''' This detects if the Escape key has been pressed, and if so closes the form.
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Private Sub AlbumSongs_KeyUp(ByVal sender As Object, ByVal e As System.Windows.Input.KeyEventArgs) Handles Me.KeyUp
        If e.Key = Key.Escape Then
            CloseForm()
        End If
    End Sub


    ''' <summary>
    ''' This catches when the user doubleclicks on a song, and raises a custom event (SongSelected) to mark the occasion
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Private Sub SongList_MouseDoubleClick(ByVal sender As Object, ByVal e As System.Windows.Input.MouseButtonEventArgs) Handles SongList.MouseDoubleClick
        RaiseEvent SongSelected(Me.CurrentSong)
    End Sub


    ''' <summary>
    ''' This catches when the form is closed, and raises SongListClosed to indicate that to the calling code.
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Private Sub AlbumSongs_Closed(ByVal sender As Object, ByVal e As System.EventArgs) Handles Me.Closed
        RaiseEvent AlbumSongsClosed()
    End Sub


    ''' <summary>
    ''' This raises the custom event SongHighlights every time the selection in the list box changes.
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Private Sub SongList_SelectionChanged(ByVal sender As Object, ByVal e As System.Windows.Controls.SelectionChangedEventArgs) Handles SongList.SelectionChanged
        RaiseEvent SongHighlighted(Me.CurrentSong)
    End Sub
#End Region



End Class
