﻿''' <summary>
''' This class encapsulates the logic that allows any Visual object to determine the
''' absolute offset between itself and the main window.
''' </summary>
''' <remarks></remarks>
Public Class Globals

    Public Shared MainWindow As Window

    ''' <summary>
    ''' This returns the relative offset (as a Point) between Globals.MainWindow and the Visual passed in.
    ''' </summary>
    ''' <param name="SourceVisual">The Visual to calculate the offset to.</param>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public Shared Function OffsetFromMainWindow(ByVal SourceVisual As Visual) As Point
        Dim MyTransform As GeneralTransform = SourceVisual.TransformToAncestor(MainWindow)
        Return MyTransform.Transform(New Point(0, 0))
    End Function


    ''' <summary>
    ''' This returns the absolute offset (as a Point) of the Visual passed in.  This value changes as the
    ''' Globals.MainWindow is moved around the screen
    ''' </summary>
    ''' <param name="SourceVisual">The Visual to calculate the offset to.</param>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public Shared Function OffsetIncludingMainWindow(ByVal SourceVisual As Visual) As Point
        Return New Point(OffsetFromMainWindow(SourceVisual).X + MainWindow.Left, _
                         OffsetFromMainWindow(SourceVisual).Y + MainWindow.Top)
    End Function
End Class
