Imports WPFMediaPlayer.BusinessObjects
Imports System.IO
Imports System.Windows.Media.Effects
Imports System.Xml.Linq
Imports System.Linq
Imports System.Windows.Media.Animation


''' <summary>
''' Interaction logic for Main.xaml
''' </summary>
''' <remarks></remarks>
Partial Public Class Main
    Inherits System.Windows.Window

    Private _PlayLists As List(Of PlayList)

    Private _MediaMetaData As PlayerMetaData

    Private WithEvents _CurrentJewelCase As JewelCase
    Private WithEvents _MediaPlayer As WMPLib.WindowsMediaPlayer
    Private _IsSongPaused As Boolean
    Private _IsSongPlaying As Boolean
    Private WithEvents _CurrentShelf As Shelf
    Private WithEvents _SongsWindow As AlbumSongs
    Private _CurrentPlayContinuity As PlayContinuitySettings
    Private _CurrentAlbum As Album
    Private _CurrentSong As Song
    Private _CurrentJewelCaseExpand As JewelCaseExpand


#Region " Enumerations "
    Private Enum PlayerModes
        DisableAll = 0
        PlayWasPressed = 1
        PauseWasPressed = 2
        StopWasPressed = 3
    End Enum

    Private Enum PlayContinuitySettings
        PlayOnce = 1
        PlayForever = 2
    End Enum
#End Region


    ''' <summary>
    ''' The default constructor.
    ''' </summary>
    ''' <remarks></remarks>
    Public Sub New()
        InitializeComponent()

        Me._PlayLists = New List(Of PlayList)

        Me.LoadMedia()
        Me.LoadPlayListsUI()
        Me.LoadShelvesUI()

        Globals.MainWindow = Me
    End Sub


#Region " Properties "
    ''' <summary>
    ''' This exposes the current selected Song.
    ''' </summary>
    ''' <value></value>
    ''' <returns>The currently selected Song object, or Nothing if no item is selected.</returns>
    ''' <remarks></remarks>
    Private ReadOnly Property CurrentSong() As Song
        Get
            If (Not IsNothing(Me._SongsWindow)) Then
                If (Not IsNothing(Me._SongsWindow.CurrentSong)) Then
                    Me._CurrentSong = Me._SongsWindow.CurrentSong
                Else
                    ' Return whatever is stored in _CurrentSong
                End If
            Else
                ' Return whatever is stored in _CurrentSong
            End If

            Return Me._CurrentSong
        End Get
    End Property


    ''' <summary>
    ''' This returns a green glow bitmap effect
    ''' </summary>
    ''' <value></value>
    ''' <returns>The glowing green bitmap effect</returns>
    ''' <remarks></remarks>
    Private ReadOnly Property ButtonGlowEffect() As BitmapEffect
        Get
            Dim GlowEffect As OuterGlowBitmapEffect
            Dim GlowColor As System.Windows.Media.Color

            GlowEffect = New OuterGlowBitmapEffect
            GlowColor = New System.Windows.Media.Color

            GlowColor.A = CType(255, Byte)
            GlowColor.B = CType(0, Byte)
            GlowColor.R = CType(0, Byte)
            GlowColor.G = CType(255, Byte)

            GlowEffect.GlowColor = GlowColor
            GlowEffect.GlowSize = 7

            Return GlowEffect
        End Get
    End Property


    ''' <summary>
    ''' This returns the currently selected playlist.
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks>
    ''' This property assumes that there is at least one playlist in the list.  This should be
    ''' relatively safe since the app won't start unless it finds one (even an empty one).
    ''' </remarks>
    Private ReadOnly Property CurrentPlayList() As PlayList
        Get
            Dim SelectedPlayList As Integer

            If (IsNothing(Me.PlayListList.SelectedItem)) Then
                SelectedPlayList = 0
            Else
                SelectedPlayList = CType(Me.PlayListList.SelectedItem, ListItem).Value
            End If
            Return Me._MediaMetaData.PlayLists(SelectedPlayList)
        End Get
    End Property


    ''' <summary>
    ''' This property exposes the internal variable _CurrentJewelCaseExpand.
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks>
    ''' This property takes care of instantiating the JewelCaseExpand object the first
    ''' time, as well as adding it to the MainGrid's Children collection.
    ''' </remarks>
    Private ReadOnly Property CurrentJewelCaseExpand() As JewelCaseExpand
        Get
            Dim AlreadyExists As Boolean

            AlreadyExists = Not IsNothing(Me._CurrentJewelCaseExpand)
            If (Not AlreadyExists) Then
                Me._CurrentJewelCaseExpand = New JewelCaseExpand()
                Me._CurrentJewelCaseExpand.Name = "MyJewelCaseExpand"
                Me.MainGrid.Children.Add(Me._CurrentJewelCaseExpand)

                ' For fun, comment the first and uncomment the second.  Then try mousing over an album.
                Me.RegisterName(Me._CurrentJewelCaseExpand.Name, Me._CurrentJewelCaseExpand)
                'Me.RegisterName(Me._CurrentJewelCaseExpand.Name, Me.MainGrid)
            End If

            Return Me._CurrentJewelCaseExpand
        End Get
    End Property
#End Region


#Region " Methods "
    ''' <summary>
    ''' This loads up the pre-defined lists of media.
    ''' </summary>
    ''' <remarks></remarks>
    Private Sub LoadMedia()
        Dim MusicBasePath As String
        Dim Answer As MsgBoxResult

        ' Retrieve the base path for the music, and make sure the directory exists
        MusicBasePath = WPFMediaPlayer.AppSettings.Core.MusicBasePath
        If (Not Directory.Exists(MusicBasePath)) Then
            MsgBox("The base music directory of '" & MusicBasePath & "' isn't available at this time, or is invalid.")
            End
        End If

        ' See if the top-level meta data is present; if so load it; if not create it and then load it
        Try
            Me._MediaMetaData = New PlayerMetaData()
            If (Not Me._MediaMetaData.Exists) Then
                Answer = MsgBox("There wasn't any meta data found.  In order to continue, a default play list " & _
                                "must be created containing all of the media found in '" & MusicBasePath & "'.  " & _
                                "Do you want to continue?", MsgBoxStyle.YesNo)
                If (Answer = MsgBoxResult.No) Then End

                Me._MediaMetaData.Initialize(True)
            End If

        Catch ex As PlayerMetaData.CeaseLoadingException
            End
        Catch ex As Exception
            Throw New Exception("An error occurred loading the playlists.", ex.InnerException)
        End Try
    End Sub



    ''' <summary>
    ''' This populates the UI with the list of playlists available
    ''' </summary>
    ''' <remarks></remarks>
    Private Sub LoadPlayListsUI()
        Dim i As Integer

        Me.PlayListList.Items.Clear()

        For i = 0 To Me._MediaMetaData.PlayLists.Count - 1
            Me.PlayListList.Items.Add(New ListItem(i, Me._MediaMetaData.PlayLists(i).Name))
        Next
    End Sub


    ''' <summary>
    ''' This refreshes the UI to display the media on the shelves
    ''' </summary>
    ''' <remarks></remarks>
    Private Sub LoadShelvesUI()
        Dim CurrentAlbumIndex As Integer

        Me.Cursor = System.Windows.Input.Cursors.Wait

        ' TODO: Allow play lists to have more than one shelf; don't hardcode in the position to 0
        Me._CurrentShelf = New Shelf(0)
        Me.MediaCabinet.Children.Clear()

        For Each CurrentAlbumIndex In Me.CurrentPlayList.Keys
            Me._CurrentShelf.AddAlbum(Me.CurrentPlayList.Item(CurrentAlbumIndex))
        Next
        AddHandler Me._CurrentShelf.AlbumOpened, AddressOf Me.OpenSongList
        AddHandler Me._CurrentShelf.AlbumSpotlighted, AddressOf Me.ShowAlbumFlyout
        AddHandler Me._CurrentShelf.AlbumUnSpotlighted, AddressOf Me.HideAlbumFlyout
        AddHandler Me._CurrentShelf.JewelCaseMoved, AddressOf Me.UpdatePlaylist
        Me.MediaCabinet.Children.Add(Me._CurrentShelf)

        Me.Cursor = System.Windows.Input.Cursors.Arrow
    End Sub


    ''' <summary>
    ''' This plays the song requested.  If there is another song being played, it will be stopped
    ''' before the new song starts.
    ''' </summary>
    ''' <remarks></remarks>
    Private Sub PlaySong()
        ' If the user hasn't selected a song, let the player figure out which one should be played next.
        If (Me.CurrentSong Is Nothing) Then
            Me.GoToNextSong()
            Return
        End If

        If (Me._MediaPlayer Is Nothing) Then
            ' This is the first time the player has been accessed, so New it up.
            Me._MediaPlayer = New WMPLib.WindowsMediaPlayer
            AddHandler Me._MediaPlayer.PlayStateChange, AddressOf GoToNextSong

            Me._MediaPlayer.URL = CurrentSong.PathToMedia


        ElseIf (Not Me._IsSongPaused Or _
                Me._MediaPlayer.URL <> CurrentSong.PathToMedia) Then
            ' The user has changed songs, or stopped the song, and wants to restart it from the beginning,
            ' or the user has switched songs (for example, pausing one song and selecting another)
            Me._MediaPlayer.controls.stop()
            Me._MediaPlayer.close()
            Me._MediaPlayer.URL = CurrentSong.PathToMedia

        Else
            ' The situation here is that there is a song loaded up in the player, it's
            ' currently paused, and it is the same as the song selected.  Nothing more than
            ' resuming play needs to happen here.
            '
            ' We don't want to set the Me._MediaPlayer.URL value here because doing so will
            ' make the .controls.play() call start at the beginning of the song.  Since we're
            ' resuming play, we want to pick up where we left off.

        End If

        ' Play (or resume) the song requested
        Me._MediaPlayer.controls.play()
        Me._IsSongPaused = False
        Me._IsSongPlaying = True

        ' Update the play controls accordingly
        Me.EnablePlayerButtons(PlayerModes.PlayWasPressed)
    End Sub


    ''' <summary>
    ''' This pauses the song currently playing, if there is one.
    ''' </summary>
    ''' <remarks></remarks>
    Private Sub PauseSong()
        If (Me._MediaPlayer Is Nothing) Then Return

        Me._MediaPlayer.controls.pause()
        Me._IsSongPaused = True
        Me._IsSongPlaying = False

        ' Update the play controls accordingly
        Me.EnablePlayerButtons(PlayerModes.PauseWasPressed)
    End Sub


    ''' <summary>
    ''' This stops the song currently playing, if there is one.
    ''' </summary>
    ''' <remarks></remarks>
    Private Sub StopSong()
        If (Me._MediaPlayer Is Nothing) Then Return

        Me._MediaPlayer.controls.stop()
        Me._IsSongPaused = False
        Me._IsSongPlaying = False

        ' Update the play controls accordingly
        Me.EnablePlayerButtons(PlayerModes.StopWasPressed)
    End Sub


    ''' <summary>
    ''' This enables/disables the player buttons as needed, based on what function was just performed.
    ''' </summary>
    ''' <param name="NewMode"></param>
    ''' <remarks></remarks>
    Private Sub EnablePlayerButtons(ByVal NewMode As PlayerModes)
        Dim PlayShouldBeEnabled, PauseShouldBeEnabled, StopShouldBeEnabled, NextShouldBeEnabled As Boolean

        PlayShouldBeEnabled = Not Me._IsSongPlaying
        Me.PlayButton.IsEnabled = PlayShouldBeEnabled
        Me.PlayButton.Opacity = IIf(PlayShouldBeEnabled, 1, 0.5)

        PauseShouldBeEnabled = (NewMode = PlayerModes.PlayWasPressed)
        Me.PauseButton.IsEnabled = PauseShouldBeEnabled
        Me.PauseButton.Opacity = IIf(PauseShouldBeEnabled, 1, 0.5)

        StopShouldBeEnabled = (NewMode = PlayerModes.PlayWasPressed) Or (NewMode = PlayerModes.PauseWasPressed)
        Me.StopButton.IsEnabled = StopShouldBeEnabled
        Me.StopButton.Opacity = IIf(StopShouldBeEnabled, 1, 0.5)

        NextShouldBeEnabled = True
        Me.NextButton.IsEnabled = NextShouldBeEnabled
        Me.NextButton.Opacity = IIf(NextShouldBeEnabled, 1, 0.5)
    End Sub


    ''' <summary>
    ''' This enables/disables the playlist buttons, based on whether or not an item was selected in the list.
    ''' </summary>
    ''' <remarks></remarks>
    Private Sub EnablePlayListButtons()
        Dim DeleteShouldBeEnabled, SelectShouldBeEnabled As Boolean

        DeleteShouldBeEnabled = Not IsNothing(Me.PlayListList.SelectedItem)
        SelectShouldBeEnabled = Not IsNothing(Me.PlayListList.SelectedItem)

        Me.DeletePlayList.IsEnabled = DeleteShouldBeEnabled
        Me.DeletePlayList.Opacity = IIf(DeleteShouldBeEnabled, 1, 0.5)

        Me.SelectPlayList.IsEnabled = SelectShouldBeEnabled
        Me.SelectPlayList.Opacity = IIf(SelectShouldBeEnabled, 1, 0.5)
    End Sub


    ''' <summary>
    ''' This updates the UI and private variables with the new PlayContinuity setting
    ''' </summary>
    ''' <param name="NewContinuity">The new PlayContinuity setting</param>
    ''' <remarks></remarks>
    Private Sub SetPlayContinuity(ByVal NewContinuity As PlayContinuitySettings)
        Select Case NewContinuity
            Case PlayContinuitySettings.PlayForever
                Me.PlayForeverButton.BitmapEffect = Me.ButtonGlowEffect
                Me.PlayOnceButton.BitmapEffect = Nothing

                Me._CurrentPlayContinuity = PlayContinuitySettings.PlayForever
            Case Else
                ' This also covers the PlayOnce option
                Me.PlayForeverButton.BitmapEffect = Nothing
                Me.PlayOnceButton.BitmapEffect = Me.ButtonGlowEffect

                Me._CurrentPlayContinuity = PlayContinuitySettings.PlayOnce
        End Select
    End Sub


    ''' <summary>
    ''' This will refresh the song list for the selected album, and selects the Song in the list, if specified.
    ''' </summary>
    ''' <param name="NewlySelectedAlbum">The newly selected album</param>
    ''' <param name="NewlySelectedSong">The newly selected song (possibly Nothing)</param>
    ''' <remarks>If the Songs Window isn't already open, this does nothing.</remarks>
    Private Sub RefreshSongList(ByVal NewlySelectedAlbum As Album, ByVal NewlySelectedSong As Song)
        If (IsNothing(Me._SongsWindow)) Then Return

        If (Not IsNothing(NewlySelectedSong)) Then Me._CurrentSong = NewlySelectedSong

        Me._SongsWindow.RefreshSongList(NewlySelectedAlbum, NewlySelectedSong)
    End Sub
#End Region


#Region " Event Handlers "
    ''' <summary>
    ''' This will load up the song list for the selected album.
    ''' </summary>
    ''' <param name="sender">The album to load.</param>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Private Sub OpenSongList(ByVal sender As Album, ByVal e As EventArgs)
        Me._CurrentAlbum = sender

        ' If the SongsWindow isn't open already, open it
        If (IsNothing(Me._SongsWindow)) Then
            Me._SongsWindow = New AlbumSongs(sender)
            AddHandler Me._SongsWindow.SongSelected, AddressOf Me.SongsWindow_SongSelected
            AddHandler Me._SongsWindow.AlbumSongsClosed, AddressOf Me.SongsWindow_AlbumSongsClosed
            AddHandler Me._SongsWindow.SongHighlighted, AddressOf Me.SongsWindow_SongHighlighted

            Me._SongsWindow.Topmost = True
            Me._SongsWindow.Show(Me)
        End If

        Me.RefreshSongList(sender, Nothing)

    End Sub


    ''' <summary>
    ''' This refreshes the main window play controls based on the state of the song window
    ''' </summary>
    ''' <param name="NewSong">The new song selected, if any</param>
    ''' <remarks></remarks>
    Private Sub SongsWindow_SongHighlighted(ByVal NewSong As Song)
        If (IsNothing(NewSong) And IsNothing(Me.CurrentSong)) Then
            Me.EnablePlayerButtons(PlayerModes.DisableAll)
            Return
        End If

        If (Not IsNothing(NewSong)) Then Me.EnablePlayerButtons(PlayerModes.StopWasPressed)
    End Sub



    ''' <summary>
    ''' This loads up the song currently selected in the player, and plays it.
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Private Sub PlayButton_Click(ByVal sender As Object, ByVal e As System.Windows.RoutedEventArgs) Handles PlayButton.Click
        Me.PlaySong()
    End Sub


    ''' <summary>
    ''' This pauses the currently playing song, if any.
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Private Sub PauseButton_Click(ByVal sender As Object, ByVal e As System.Windows.RoutedEventArgs) Handles PauseButton.Click
        Me.PauseSong()
    End Sub


    ''' <summary>
    ''' This stops the current song being played, if any.
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Private Sub StopButton_Click(ByVal sender As Object, ByVal e As System.Windows.RoutedEventArgs) Handles StopButton.Click
        Me.StopSong()
    End Sub


    ''' <summary>
    ''' If the user presses the Escape key, this handler will either close the Songs pop up
    ''' window (if open), or the main window (if it is not).
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks>
    ''' If the Songs window exists then assume it is open and close it.  This is a safe
    ''' assumption since we detect when the Songs window closes, and nuke Me._SongsWindow at that
    ''' time (via the SongsWindow_AlbumSongsClosed handler).
    ''' 
    ''' This ultimately is a workaround for my seeming inability to set the focus on the popup window
    ''' programmatically when I open it.  If I could do that, the pop up window has a handler similiar
    ''' to this one, and closes itself when the user hits the ESC key.
    ''' </remarks>
    Private Sub Main_KeyUp(ByVal sender As Object, ByVal e As System.Windows.Input.KeyEventArgs) Handles Me.KeyUp
        If (e.Key = Key.Escape) Then
            If (Not IsNothing(Me._SongsWindow)) Then
                Me._SongsWindow.Close()
            Else
                Me.Close()
            End If
        End If
    End Sub


    ''' <summary>
    ''' This initializes the form controls.
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Private Sub Main_Loaded(ByVal sender As Object, ByVal e As System.Windows.RoutedEventArgs) Handles Me.Loaded
        Me.EnablePlayerButtons(PlayerModes.DisableAll)
        Me.EnablePlayListButtons()
        Me.SetPlayContinuity(PlayContinuitySettings.PlayForever)
    End Sub


    ''' <summary>
    ''' This is designed to be an event handler that is attached at runtime.  This attempts to play the new song.
    ''' </summary>
    ''' <param name="NewSong">The new song selected</param>
    ''' <remarks></remarks>
    Private Sub SongsWindow_SongSelected(ByVal NewSong As Song)
        Me.PlaySong()
    End Sub


    ''' <summary>
    ''' If the Shuffle button is Checked, this will make it glow.  If it is not Checked, it will clear out the glow effect.
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Private Sub ShuffleButton_Click(ByVal sender As Object, ByVal e As System.Windows.RoutedEventArgs) Handles ShuffleButton.Click
        Me.ShuffleButton.BitmapEffect = IIf(Me.ShuffleButton.IsChecked, Me.ButtonGlowEffect, Nothing)
    End Sub


    ''' <summary>
    ''' This toggles the play continuity buttons to the PlayForever button
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Private Sub PlayForeverButton_Click(ByVal sender As Object, ByVal e As System.Windows.RoutedEventArgs) Handles PlayForeverButton.Click
        Me.SetPlayContinuity(PlayContinuitySettings.PlayForever)
    End Sub


    ''' <summary>
    ''' This toggles the play continuity buttons to the PlayOnce button
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Private Sub PlayOnceButton_Click(ByVal sender As Object, ByVal e As System.Windows.RoutedEventArgs) Handles PlayOnceButton.Click
        Me.SetPlayContinuity(PlayContinuitySettings.PlayOnce)
    End Sub


    ''' <summary>
    ''' This handler monitors state changes in the song being played, and looks for the end of the current song.
    ''' </summary>
    ''' <param name="NewState">The new state</param>
    ''' <remarks>
    ''' If the user didn't have a album/song previously selected, this method will pick one for the user.  Which
    ''' album/song is selected depends on the Shuffle setting.  If the user has Shuffle selected, it picks the
    ''' album and song at random.  If they didn't have Shuffle selected, it picks the first song on the first
    ''' album.
    ''' </remarks>
    Private Sub GoToNextSong(ByVal NewState As Long)
        Dim PositionOfSongJustPlayed, PositionOfSongToBePlayed As Integer
        Dim PositionOfAlbumJustPlayed, PositionOfAlbumToBePlayed As Integer


        ' A state of "8" indicates that that the current song has ended; we don't care about anything else
        If (NewState <> 8) Then Return

        ' If the user has the player set to play only the current song, then just return
        If (Me._CurrentPlayContinuity = PlayContinuitySettings.PlayOnce) Then Return


        ' This should make sure there is at least one album in the play list, and one song on that album
        If (Me.CurrentPlayList.Count = 0) Then
            MsgBox("There are no albums found in this playlist.")
            Return
        End If


        ' The user has selected to have the player automatically go to the next song
        If (Me.ShuffleButton.IsChecked) Then
            ' Play a random song from a random album
            PositionOfAlbumToBePlayed = (New Random).Next(0, Me.CurrentPlayList.Count - 1)
            PositionOfSongToBePlayed = (New Random).Next(0, Me.CurrentPlayList.Item(PositionOfAlbumToBePlayed).Songs.Count - 1)

        Else
            If (IsNothing(Me._CurrentAlbum)) Then
                ' The user didn't have a song playing previously, so just play the first song on the first album
                PositionOfAlbumToBePlayed = 0
                PositionOfSongToBePlayed = 0
            Else
                ' Play the next song in order
                PositionOfAlbumJustPlayed = Me._CurrentAlbum.Index
                PositionOfAlbumToBePlayed = -1
                PositionOfSongJustPlayed = Me.CurrentSong.Index
                PositionOfSongToBePlayed = -1

                If (PositionOfSongJustPlayed < Me._CurrentAlbum.Songs.Count - 1) Then
                    ' If we're in the middle of an album, just go to the next song on the same album
                    PositionOfAlbumToBePlayed = PositionOfAlbumJustPlayed
                    PositionOfSongToBePlayed = PositionOfSongJustPlayed + 1

                ElseIf (PositionOfSongJustPlayed = Me._CurrentAlbum.Songs.Count - 1) Then
                    ' If we're at the end of the album, go to the next album
                    If (PositionOfAlbumJustPlayed < Me.CurrentPlayList.Count - 1) Then
                        ' If we're in the middle of the playlist, just go to the next album
                        PositionOfAlbumToBePlayed = PositionOfAlbumJustPlayed + 1

                    ElseIf (PositionOfAlbumJustPlayed = Me.CurrentPlayList.Count - 1) Then
                        ' If we're at the end of the playlist, go to the first album
                        PositionOfAlbumToBePlayed = 0
                    End If

                    ' Regardless of which album we pick, we're playing the first song on it
                    PositionOfSongToBePlayed = 0
                End If
            End If
        End If

        ' Spin up the new song to be played
        Me._CurrentAlbum = Me.CurrentPlayList.Item(PositionOfAlbumToBePlayed)
        Me._CurrentSong = Me._CurrentAlbum.Songs.Item(PositionOfSongToBePlayed)
        Me.RefreshSongList(Me._CurrentAlbum, Me._CurrentSong)
        Me.PlaySong()
    End Sub


    ''' <summary>
    ''' This variant forces the player to go to the next song.
    ''' </summary>
    ''' <remarks>
    ''' It calls the other variant, and passes it "8", which is the state
    ''' that indicates that the current song has completed playing.
    ''' </remarks>
    Private Sub GoToNextSong()
        Me.GoToNextSong(8)
    End Sub


    ''' <summary>
    ''' This clears out the handle to the AlbumSongs window when it is closed
    ''' </summary>
    ''' <remarks></remarks>
    Private Sub SongsWindow_AlbumSongsClosed()
        Me._SongsWindow = Nothing
    End Sub


    ''' <summary>
    ''' This ensures that if the Songs Window popup is still open when the main form is closed, it gets closed too.
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Private Sub Main_Closing(ByVal sender As Object, ByVal e As System.ComponentModel.CancelEventArgs) Handles Me.Closing
        If (IsNothing(Me._SongsWindow)) Then Return

        Try
            Me._SongsWindow.Close()
        Catch ex As Exception
            ' If an error is throw, then it means that the window was already closed.
        Finally
            Me._SongsWindow = Nothing
        End Try
    End Sub


    ''' <summary>
    ''' This reloads the Media Cabinet with the contents of the selected playlist.
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Private Sub PlayListList_MouseDoubleClick(ByVal sender As Object, ByVal e As System.Windows.Input.MouseButtonEventArgs) Handles PlayListList.MouseDoubleClick
        Me.LoadShelvesUI()
    End Sub


    ''' <summary>
    ''' This reloads the Media Cabinet with the contents of the selected playlist.
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Private Sub SelectPlayList_Click(ByVal sender As Object, ByVal e As System.Windows.RoutedEventArgs) Handles SelectPlayList.Click
        Me.LoadShelvesUI()
    End Sub


    ''' <summary>
    ''' This forces the player to go on to the next song
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Private Sub NextButton_Click(ByVal sender As Object, ByVal e As System.Windows.RoutedEventArgs) Handles NextButton.Click
        Me.GoToNextSong()
    End Sub


    ''' <summary>
    ''' After the user has made a selection, refresh the buttons accordingly.
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Private Sub PlayListList_SelectionChanged(ByVal sender As Object, ByVal e As System.Windows.Controls.SelectionChangedEventArgs) Handles PlayListList.SelectionChanged
        Me.EnablePlayListButtons()
    End Sub


    ''' <summary>
    ''' This prompts the user for the name for the new playlist, and then creates it.
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Private Sub AddPlayList_Click(ByVal sender As Object, ByVal e As System.Windows.RoutedEventArgs) Handles AddPlayList.Click
        Dim NewPlayListName As String
        Dim PromptForm As PlaylistNamePrompt


        Try
            Me.Cursor = System.Windows.Input.Cursors.Wait

            ' Prompt for the user for the new playlist name
            PromptForm = New PlaylistNamePrompt
            PromptForm.ShowDialog(Me)
            NewPlayListName = PromptForm.NameSpecified
            PromptForm = Nothing
            If (NewPlayListName.Length > 0) Then
                ' Create and load up the new, blank, playlist
                Me._MediaMetaData.AddNewPlayList(NewPlayListName)
                Me.LoadPlayListsUI()
            End If

        Catch ex As PlayerMetaData.PlaylistAlreadyExistsException
            MsgBox("That playlist name is already in use.  Please enter another.")
        Catch ex As Exception
            Throw ex
        Finally
            Me.Cursor = System.Windows.Input.Cursors.Arrow
        End Try
    End Sub


    ''' <summary>
    ''' This prompts the user to confirm deletion of the selected playlist, and if they confirm it will remove it and refresh the UI.
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Private Sub DeletePlayList_Click(ByVal sender As Object, ByVal e As System.Windows.RoutedEventArgs) Handles DeletePlayList.Click
        Dim PlayListName As String
        Dim WasPlayListDeleted As Boolean

        PlayListName = Me.CurrentPlayList.Name
        If (MsgBox("Do you really want to delete the '" & PlayListName & "' playlist?", MsgBoxStyle.YesNo) = MsgBoxResult.No) Then Return

        Try
            WasPlayListDeleted = Me._MediaMetaData.DeletePlayList(PlayListName)
            If (WasPlayListDeleted) Then Me.LoadPlayListsUI()

        Catch ex As PlayerMetaData.DefaultPlayListCannotBeDeletedException
            MsgBox("You are not allowed to remove the default playlist '" & PlayListName & "'")
        Catch ex As Exception
            Throw ex
        End Try
    End Sub


    ''' <summary>
    ''' This configures the CurrentJewelCaseExpand for the album selected.
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Private Sub ShowAlbumFlyout(ByVal sender As JewelCase, ByVal e As JewelCaseHighlightArgs)
        Me.CurrentJewelCaseExpand.JacketArt = sender.JacketArt
        Me.CurrentJewelCaseExpand.VisibleWithAnimation = True
        Me.CurrentJewelCaseExpand.HorizontalAlignment = Windows.HorizontalAlignment.Left
        Me.CurrentJewelCaseExpand.VerticalAlignment = Windows.VerticalAlignment.Top
        Me.CurrentJewelCaseExpand.Margin = New Thickness(e.JewelCaseBottomLeftCorner.X, e.JewelCaseBottomLeftCorner.Y - 20, 0, 0)

        ' TODO: Programmatically tweak the top Margin of the user control to give the effect of sliding up.
        ' TOOO: Ideally this would be encapsulated into the VisibleWithAnimation property, and the control would take care of it internally
        Dim MyStoryBoard As Storyboard
        Dim SlideUpAnimation As ThicknessAnimation
        ' By not specifying the starting location in the second animation, it remembers where the last ending location of the
        ' object is, and uses that as it's start.  This is partially because I am not destroying the control when it is hidden,
        ' I am only turning it invisible.  This has the effect of the usercontrol chasing the mousing around the screen.
        '
        ' Additionally, specifying the Me.CurrentJewelCaseExpand.Margin property as the first parameter to the constructor isn't
        ' enough.  You need to create a completely fresh Thickness object to get this to work.
        SlideUpAnimation = New Animation.ThicknessAnimation(New Thickness(e.JewelCaseBottomLeftCorner.X, e.JewelCaseBottomLeftCorner.Y - 20, 0, 0), _
                                                            New Thickness(e.JewelCaseBottomLeftCorner.X, e.JewelCaseBottomLeftCorner.Y + 5, 0, 0), _
                                                            New Duration(New TimeSpan(0, 0, 0, 0, 200)))
        'SlideUpAnimation = New Animation.ThicknessAnimation(New Thickness(e.JewelCaseBottomLeftCorner.X, e.JewelCaseBottomLeftCorner.Y + 5, 0, 0), _
        '                                                    New Duration(New TimeSpan(0, 0, 0, 0, 200)))


        MyStoryBoard = New Storyboard
        MyStoryBoard.Children.Add(SlideUpAnimation)

        Storyboard.SetTargetName(SlideUpAnimation, Me.CurrentJewelCaseExpand.Name)
        Storyboard.SetTargetProperty(SlideUpAnimation, New PropertyPath(JewelCaseExpand.MarginProperty))
        ' TODO: Remove this if not needed
        'sb.Completed += new EventHandler(sb_Completed);
        MyStoryBoard.Begin(Me.MainGrid)
    End Sub


    ''' <summary>
    ''' This hides the CurrentJewelCaseExpand object.
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <remarks></remarks>
    Private Sub HideAlbumFlyout(ByVal sender As JewelCase)
        If (Not IsNothing(Me._CurrentJewelCaseExpand)) Then
            Me._CurrentJewelCaseExpand.VisibleWithAnimation = False
        End If
    End Sub


    ''' <summary>
    ''' This syncs the current Playlist up to the new Shelf configuration, and saves those changes to the persistent medium.
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="SourceIndex"></param>
    ''' <param name="DestinationIndex"></param>
    ''' <remarks></remarks>
    Private Sub UpdatePlaylist(ByVal sender As Shelf, ByVal SourceIndex As Integer, ByVal DestinationIndex As Integer)
        Dim CurrentJewelCase As JewelCase

        Me.CurrentPlayList.Clear()

        For Each CurrentJewelCase In sender.JewelCases
            Me.CurrentPlayList.Add(CurrentJewelCase.Index, CurrentJewelCase.SourceAlbum)
        Next

        Me.CurrentPlayList.Save()
    End Sub
#End Region

End Class
