﻿Partial Public Class PlaylistNamePrompt

    ''' <summary>
    ''' This returns what was entered into the text box.
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public ReadOnly Property NameSpecified() As String
        Get
            Return Me.PlaylistName.Text.Trim
        End Get
    End Property

#Region " Methods "
    ''' <summary>
    ''' This is used to shut the form down after the user has opted to cancel out of creating the new playlist.
    ''' </summary>
    ''' <remarks></remarks>
    Private Sub CloseFormOnCancel()
        Me.PlaylistName.Text = ""
        Me.Close()
    End Sub


    ''' <summary>
    ''' This overloaded routine allows the calling window to pass in a reference to itself so that the current
    ''' window can position itself relative to it.
    ''' </summary>
    ''' <param name="ParentForm"></param>
    ''' <remarks></remarks>
    Public Overloads Sub ShowDialog(ByVal ParentForm As System.Windows.Window)
        Me.Left = ParentForm.Left + 50
        Me.Top = ParentForm.Top + 50
        MyBase.ShowDialog()
    End Sub
#End Region


#Region " Event Handlers "
    ''' <summary>
    ''' This makes sure the user entered something into the text box, and if so closes it.
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Private Sub OKButton_Click(ByVal sender As System.Object, ByVal e As System.Windows.RoutedEventArgs) Handles OKButton.Click
        If (Me.PlaylistName.Text.Trim.Length = 0) Then
            MsgBox("Please enter a new playlist name.")
            Me.PlaylistName.Focus()
        Else
            Me.Close()
        End If
    End Sub


    ''' <summary>
    ''' This clears out the textbox, and closes the form.
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Private Sub CancelButton_Click(ByVal sender As Object, ByVal e As System.Windows.RoutedEventArgs) Handles CancelButton.Click
        Me.CloseFormOnCancel()
    End Sub


    ''' <summary>
    ''' This sets the focus to the PlaylistName textbox
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Private Sub PlaylistNamePrompt_Loaded(ByVal sender As Object, ByVal e As System.Windows.RoutedEventArgs) Handles Me.Loaded
        Me.PlaylistName.Focus()
    End Sub


    ''' <summary>
    ''' This allows the user to move the window around.
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Private Sub PlaylistNamePrompt_MouseDown(ByVal sender As Object, ByVal e As System.Windows.Input.MouseButtonEventArgs) Handles Me.MouseDown
        Me.DragMove()
    End Sub



    ''' <summary>
    ''' This watches for the ESC key to be pressed, and closes the form.
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks>
    ''' This is a hack to get around interfering with the main form's custom way of handling form closures.  The main form
    ''' and the AlbumSongs form are designed to be open, side by side.  However, regardless of where the focus is, I want
    ''' the AlbumSongs window to close first if it is open, and ONLY that form.  If that form isn't open, then the main form
    ''' can close.
    ''' 
    ''' I call this a hack because if I try to set the Cancel button's IsCancel property to True, the dialog closes, but
    ''' the main form also catches the ESC keypress, and closes with it.  I haven't found a way to interrupt this keypress
    ''' from being sent through to the main form.
    ''' </remarks>
    Private Sub PlaylistNamePrompt_KeyUp(ByVal sender As Object, ByVal e As System.Windows.Input.KeyEventArgs) Handles Me.KeyUp
        If (e.Key = Key.Escape) Then
            e.Handled = True
            Me.CloseFormOnCancel()
        End If
    End Sub
#End Region



End Class
